@@ -29,7 +29,7 @@ from six.moves import urllib
 from . import requests_usbmux, xcui_element_types
 from ._proto import *
 from .exceptions import *
-from .utils import inject_call
+from .utils import inject_call, limit_call_depth
 
 try:
     from functools import cached_property  # Python3.8+
@@ -56,13 +56,9 @@ PORTRAIT = 'PORTRAIT'
 LANDSCAPE_RIGHT = 'UIA_DEVICE_ORIENTATION_LANDSCAPERIGHT'
 PORTRAIT_UPSIDEDOWN = 'UIA_DEVICE_ORIENTATION_PORTRAIT_UPSIDEDOWN'
 
-JSONDecodeError = json.decoder.JSONDecodeError if hasattr(
-    json.decoder, "JSONDecodeError") else ValueError
-
 
 class Status(enum.IntEnum):
     # 不是怎么准确，status在mds平台上变来变去的
-    INVALID_SESSION_ID = 10  # error: "invalid session id", "message": "Session does not exist"
     UNKNOWN = 100  # other status
     ERROR = 110
 
@@ -133,19 +129,6 @@ def _requests_session_pool_get(scheme, netloc):
 def _is_tmq_platform() -> bool:
     return os.getenv("TMQ") == "true"
 
-
-def _is_session_id_error(value: Union[str, dict]):
-    if isinstance(value, dict):  # 新版WDA逻辑
-        error = value.get('error')
-        if error == "invalid session id" or "possibly crashed" in value.get(
-                'message', ''):
-            return True
-    elif isinstance(value, str):  # 旧版WDA中的value就是字符串
-        if "invalid session id" in value or "possibly crashed" in value:
-            return True
-    return False
-
-
 def _unsafe_httpdo(url, method='GET', data=None):
     """
     Do HTTP Request
@@ -163,8 +146,7 @@ def _unsafe_httpdo(url, method='GET', data=None):
                                            url,
                                            json=data,
                                            timeout=HTTP_TIMEOUT)
-    except (requests.exceptions.ConnectionError,
-            requests.exceptions.ReadTimeout) as e:
+    except (requests.ConnectionError, requests.ReadTimeout) as e:
         raise
 
     if response.status_code == 502:  # Bad Gateway
@@ -179,18 +161,15 @@ def _unsafe_httpdo(url, method='GET', data=None):
         retjson['status'] = retjson.get('status', 0)
         r = convert(retjson)
 
-        if r.status != 0:  # in new WDA, status is always 0
-            if _is_session_id_error(r.value):
-                r['status'] = Status.INVALID_SESSION_ID
-            raise WDARequestError(r.status, r.value)
-
         if isinstance(r.value, dict) and r.value.get("error"):
-            if _is_session_id_error(r.value):
-                status = Status.INVALID_SESSION_ID
-            else:
-                status = Status.ERROR
+            status = Status.ERROR
             value = r.value.copy()
             value.pop("traceback", None)
+
+            for errCls in (WDAInvalidSessionIdError, WDAPossiblyCrashedError, WDAKeyboardNotPresentError, WDAUnknownError):
+                if errCls.check(value):
+                    raise errCls(status, value)
+
             raise WDARequestError(status, value)
         return r
     except JSONDecodeError:
@@ -261,18 +240,21 @@ class BaseClient(object):
         self.__session_id = _session_id
         self.__is_app = bool(_session_id)  # set to freeze session_id
         self.__timeout = 30.0
-        self.__target = None
         self.__callbacks = defaultdict(list)
         self.__callback_depth = 0
-        self._init_fix()
+        self.__callback_running = False
+
+        if not _session_id:
+            self._init_callback()
 
     def _callback_fix_invalid_session_id(self, err: WDAError):
         """ 当遇到 invalid session id错误时，更新session id并重试 """
-        if isinstance(err, WDARequestError) and \
-                err.status == Status.INVALID_SESSION_ID: # and not self.__is_app:
-            self.close()
+        if isinstance(err, WDAInvalidSessionIdError): # and not self.__is_app:
             self.session_id = None
             return Callback.RET_RETRY
+        if isinstance(err, WDAPossiblyCrashedError):
+            self.session_id = self.session().session_id # generate new sessionId
+            return Callback.RET_RETRY
         """ 等待设备恢复上线 """
 
     def _callback_wait_ready(self, err):
@@ -300,7 +282,7 @@ class BaseClient(object):
             method=method.upper(), body=body or '', url=url))
         logger.warning("Error: %s", err)
 
-    def _init_fix(self):
+    def _init_callback(self):
         self.register_callback(Callback.ERROR,
                                self._callback_fix_invalid_session_id)
         if _is_tmq_platform():
@@ -313,6 +295,24 @@ class BaseClient(object):
             self.register_callback(Callback.ERROR,
                                    self._callback_tmq_print_error)
 
+    def _callback_json_report(self, method, urlpath):
+        """ TODO: ssx """
+        pass
+
+    def _set_output_report(self, filename: str):
+        """
+        Args:
+            filename: json log
+        """
+        self.register_callback(Callback.HTTP_REQUEST_BEFORE, self._callback_json_report)
+
+    def is_ready(self) -> bool:
+        try:
+            self.status()
+            return True
+        except:
+            return False
+
     def wait_ready(self, timeout=120, noprint=False) -> bool:
         """
         wait until WDA back to normal
@@ -325,15 +325,14 @@ class BaseClient(object):
         def _dprint(message: str):
             if noprint:
                 return
-            print(message)
+            print(time.ctime(), message)
 
         _dprint("Wait ready (timeout={:.1f})".format(timeout))
         while time.time() < deadline:
-            try:
-                self.status()
+            if self.is_ready():
                 _dprint("device back online")
                 return True
-            except:
+            else:
                 _dprint("wait_ready left {:.1f} seconds".format(deadline -
                                                                 time.time()))
                 time.sleep(1.0)
@@ -366,18 +365,24 @@ class BaseClient(object):
         """ 运行回调函数 """
         if not callbacks:
             return
-        for fn in callbacks[event_name]:
-            ret = inject_call(fn, **kwargs)
-            if ret in [
-                    Callback.RET_RETRY, Callback.RET_ABORT,
-                    Callback.RET_CONTINUE
-            ]:
-                return ret
+
+        self.__callback_running = True
+        try:
+            for fn in callbacks[event_name]:
+                ret = inject_call(fn, **kwargs)
+                if ret in [
+                        Callback.RET_RETRY, Callback.RET_ABORT,
+                        Callback.RET_CONTINUE
+                ]:
+                    return ret
+        finally:
+            self.__callback_running = False
 
     @property
     def callbacks(self):
         return self.__callbacks
 
+    @limit_call_depth(4)
     def _fetch(self,
                method: str,
                urlpath: str,
@@ -386,12 +391,13 @@ class BaseClient(object):
         """ do http request """
         urlpath = "/" + urlpath.lstrip("/")  # urlpath always startswith /
 
-        callbacks = None
-        if self.__callback_depth == 0:
-            callbacks = self.__callbacks
+        callbacks = self.__callbacks
+
+        if self.__callback_running:
+            callbacks = None
 
-        self.__callback_depth += 1
         url = urljoin(self.__wda_url, urlpath)
+
         run_callback = functools.partial(self._run_callback,
                                          callbacks=callbacks,
                                          method=method,
@@ -417,8 +423,6 @@ class BaseClient(object):
                 return
             else:
                 raise
-        finally:
-            self.__callback_depth -= 1
 
     @property
     def http(self):
@@ -459,6 +463,11 @@ class BaseClient(object):
         """ unlock screen, double press home """
         return self.http.post('/wda/unlock')
 
+    def sleep(self, secs: float):
+        """ same as time.sleep """
+        time.sleep(secs)
+
+    @retry.retry(WDAUnknownError, tries=3, delay=.5, jitter=.2)
     def app_current(self) -> dict:
         """
         Returns:
@@ -596,13 +605,16 @@ class BaseClient(object):
             res = self.session().app_state(bundle_id)
             if res.value != 4:
                 raise
-        return Client(self.__wda_url, _session_id=res.sessionId)
+        client = Client(self.__wda_url, _session_id=res.sessionId)
+        client.__timeout = self.__timeout
+        client.__callbacks = self.__callbacks
+        return client
 
     def close(self):  # close session
         try:
             return self._session_http.delete('/')
         except WDARequestError as e:
-            if e.status != Status.INVALID_SESSION_ID:
+            if not isinstance(e, (WDAInvalidSessionIdError, WDAPossiblyCrashedError)):
                 raise
 
     #@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@#
@@ -820,7 +832,8 @@ class BaseClient(object):
                                        dict(duration=duration))
 
     def tap(self, x, y):
-        if _is_tmq_platform() and os.environ.get("TMQ_ORIGIN") == "civita":  # in TMQ and belong to MDS
+        if _is_tmq_platform() and os.environ.get(
+                "TMQ_ORIGIN") == "civita":  # in TMQ and belong to MDS
             return self._session_http.post("/mds/touchAndHold",
                                            dict(x=x, y=y, duration=0.02))
         return self._session_http.post('/wda/tap/0', dict(x=x, y=y))
@@ -881,6 +894,13 @@ class BaseClient(object):
         data = dict(fromX=x1, fromY=y1, toX=x2, toY=y2, duration=duration)
         return self._session_http.post('/wda/dragfromtoforduration', data=data)
 
+    def _fast_swipe(self, x1, y1, x2, y2, velocity: int = 500):
+        """
+        velocity: the larger the faster
+        """
+        data = dict(fromX=x1, fromY=y1, toX=x2, toY=y2, velocity=velocity)
+        return self._session_http.post('/wda/drag', data=data)
+
     def swipe_left(self):
         """ swipe right to left """
         w, h = self.window_size()
@@ -901,13 +921,27 @@ class BaseClient(object):
         w, h = self.window_size()
         return self.swipe(w // 2, h // 2, w // 2, h - 1)
 
+    def _fast_swipe_ext(self, direction: str):
+        if direction == "up":
+            w, h = self.window_size()
+            return self.swipe(w // 2, h // 2, w // 2, 1)
+        elif direction == "down":
+            w, h = self.window_size()
+            return self._fast_swipe(w // 2, h // 2, w // 2, h - 1)
+        else:
+            raise RuntimeError("not supported direction:", direction)
+
     @property
     def orientation(self):
         """
         Return string
         One of <PORTRAIT | LANDSCAPE>
         """
-        return self._session_http.get('orientation').value
+        for _ in range(3):
+            result = self._session_http.get('orientation').value
+            if result:
+                return result
+            time.sleep(.5)
 
     @orientation.setter
     def orientation(self, value):
@@ -925,11 +959,23 @@ class BaseClient(object):
             namedtuple: eg
                 Size(width=320, height=568)
         """
+        w, h = self._unsafe_window_size()
+        if w == 0 and h == 0:
+            # FIXME(ssx): launch another app is not a good idea, but I did'nt found some other way
+            self.unlock()
+            self.session("com.apple.Preferences")
+        return self._unsafe_window_size()
+
+    def _unsafe_window_size(self):
+        """
+        returns (width, height) might be (0, 0)
+        """
         value = self._session_http.get('/window/size').value
         w = roundint(value['width'])
         h = roundint(value['height'])
         return namedtuple('Size', ['width', 'height'])(w, h)
 
+    @retry.retry(WDAKeyboardNotPresentError, tries=3, delay=1.0)
     def send_keys(self, value):
         """
         send keys, yet I know not, todo function
@@ -999,6 +1045,8 @@ class BaseClient(object):
 
 
 class Alert(object):
+    DEFAULT_ACCEPT_BUTTONS = []
+
     def __init__(self, client: BaseClient):
         self._c = client
         self.http = client._session_http
@@ -1051,10 +1099,12 @@ class Alert(object):
         raise ValueError("Only these buttons can be clicked", avaliable_names)
 
     @contextlib.contextmanager
-    def watch_and_click(
-            self,
-            buttons: Optional[list] = ["使用App时允许", "好", "稍后", "稍后提醒", "确定", "允许", "以后"],
-            interval: float =2.0):
+    def watch_and_click(self,
+                        buttons: Optional[list] = [
+                            "使用App时允许", "无线局域网与蜂窝网络", "好", "稍后", "稍后提醒", "确定",
+                            "允许", "以后", "打开", "录屏"
+                        ],
+                        interval: float = 2.0):
         """ watch and click button
         Args:
             buttons: buttons name which need to click
@@ -1470,7 +1520,7 @@ class Element(object):
     @property
     def info(self):
         return {
-            "id": self.id,
+            "id": self.session_id,
             "label": self.label,
             "value": self.value,
             "text": self.text,
@@ -1622,3 +1672,20 @@ class USBClient(Client):
     """ connect device through unix:/var/run/usbmuxd """
     def __init__(self, udid: str = "", port: int = 8100):
         super().__init__(url="usbmux://{}:{}".format(udid, port))
+
+        if not self.is_ready():
+            self._start_xctest()
+
+    def _start_xctest(self) -> bool:
+        import shutil
+        import subprocess
+        tins_path = shutil.which("tins")
+        if not tins_path:
+            return False
+
+        logger.info("WDA is not running, exec: tins xctest")
+        p = subprocess.Popen([tins_path, 'xctest'])
+        time.sleep(3)
+        assert p.poll() is None
+        return self.wait_ready()
+
